@@ -1,12 +1,11 @@
 import asyncio
-import datetime
 import json
 from copy import copy
-from typing import Literal, cast
+from typing import cast
 
+import httpx
 import json_numpy  # type: ignore
 import numpy as np
-from dateutil import parser  # type: ignore
 from fastapi import (
     APIRouter,
     BackgroundTasks,
@@ -25,36 +24,36 @@ from phosphobot.hardware.base import BaseManipulator
 from phosphobot.leader_follower import RobotPair, leader_follower_loop
 from phosphobot.models import (
     AIControlStatusResponse,
-    AIStatusRequest,
     AIStatusResponse,
     AppControlData,
     CalibrateResponse,
     EndEffectorPosition,
+    EndEffectorReadRequest,
     FeedbackRequest,
     JointsReadRequest,
     JointsReadResponse,
     JointsWriteRequest,
     MoveAbsoluteRequest,
     RelativeEndEffectorPosition,
+    RobotConfigResponse,
     RobotConnectionRequest,
     SpawnStatusResponse,
     StartAIControlRequest,
     StartLeaderArmControlRequest,
     StartServerRequest,
     StatusResponse,
+    TemperatureReadResponse,
+    TemperatureWriteRequest,
     TorqueControlRequest,
     TorqueReadResponse,
     UDPServerInformationResponse,
     VoltageReadResponse,
-    TemperatureReadResponse,
-    TemperatureWriteRequest,
-    RobotConfigResponse,
 )
 from phosphobot.robot import (
+    PiperHardware,
     RemotePhosphobot,
     RobotConnectionManager,
     SO100Hardware,
-    PiperHardware,
     get_rcm,
 )
 from phosphobot.supabase import get_client, user_is_logged_in
@@ -64,7 +63,7 @@ from phosphobot.teleoperation import (
     get_teleop_manager,
     get_udp_server,
 )
-from phosphobot.utils import background_task_log_exceptions
+from phosphobot.utils import background_task_log_exceptions, get_tokens
 
 # This is used to send numpy arrays as JSON to OpenVLA server
 json_numpy.patch()
@@ -73,10 +72,10 @@ router = APIRouter(tags=["control"])
 
 
 # Object that controls the global /auto, /gravity state in a thread safe way
-ai_control_signal = CustomAIControlSignal()
-gravity_control = ControlSignal()
-leader_follower_control = ControlSignal()
-vr_control_signal = ControlSignal()
+signal_ai_control = CustomAIControlSignal()
+signal_gravity_control = ControlSignal()
+signal_leader_follower = ControlSignal()
+signal_vr_control = ControlSignal()
 
 
 @router.post(
@@ -126,7 +125,7 @@ async def move_teleop_ws(
 
     await websocket.accept()
 
-    vr_control_signal.start()
+    signal_vr_control.start()
     try:
         while True:
             data = await websocket.receive_text()
@@ -140,7 +139,7 @@ async def move_teleop_ws(
     except WebSocketDisconnect:
         logger.warning("WebSocket client disconnected")
 
-    vr_control_signal.stop()
+    signal_vr_control.stop()
 
 
 @router.post("/move/teleop/udp", response_model=UDPServerInformationResponse)
@@ -477,6 +476,7 @@ async def move_sleep(
     description="Retrieve the position, orientation, and open status of the robot's end effector. Only available for manipulators.",
 )
 async def end_effector_read(
+    query: EndEffectorReadRequest | None = None,
     robot_id: int = 0,
     rcm: RobotConnectionManager = Depends(get_rcm),
 ) -> EndEffectorPosition:
@@ -484,6 +484,8 @@ async def end_effector_read(
     Get the position, orientation, and open status of the end effector.
     """
     robot = await rcm.get_robot(robot_id)
+    if query is None:
+        query = EndEffectorReadRequest(sync=False)
 
     if not isinstance(robot, BaseManipulator):
         raise HTTPException(
@@ -499,7 +501,7 @@ async def end_effector_read(
             detail=f"Call /move/init before using this endpoint for robot {robot.name}. ",
         )
 
-    position, orientation, open_status = robot.get_end_effector_state()
+    position, orientation, open_status = robot.get_end_effector_state(sync=query.sync)
     # Remove the initial position and orientation (used to zero the robot)
     position = position - initial_position
     orientation = orientation - initial_orientation_rad
@@ -809,7 +811,7 @@ async def start_leader_follower(
     The first robot is the leader with gravity compensation enabled,
     and the second robot follows the leader's joint positions.
     """
-    if leader_follower_control.is_in_loop():
+    if signal_leader_follower.is_in_loop():
         raise HTTPException(
             status_code=400,
             detail="Leader-follower control is already running. Call /move/leader/stop first.",
@@ -864,13 +866,13 @@ async def start_leader_follower(
         robot_pairs.append(RobotPair(leader=leader, follower=follower))
 
     # Create control signal for managing the leader-follower operation
-    leader_follower_control.start()
+    signal_leader_follower.start()
 
     # Add background task to run the control loop
     background_tasks.add_task(
         background_task_log_exceptions(leader_follower_loop),
         robot_pairs=robot_pairs,
-        control_signal=leader_follower_control,
+        control_signal=signal_leader_follower,
         invert_controls=request.invert_controls,
         enable_gravity_compensation=request.enable_gravity_compensation,
         compensation_values=request.gravity_compensation_values,
@@ -891,12 +893,12 @@ async def stop_leader_follower(
     """
     Stop the leader-follower
     """
-    if not leader_follower_control.is_in_loop():
+    if not signal_leader_follower.is_in_loop():
         return StatusResponse(
             status="error", message="Leader-follower control is not running"
         )
 
-    leader_follower_control.stop()
+    signal_leader_follower.stop()
     return StatusResponse(message="Stopping leader-follower control")
 
 
@@ -909,7 +911,7 @@ async def start_gravity(
     """
     Enable gravity compensation for the robot.
     """
-    if gravity_control.is_in_loop():
+    if signal_gravity_control.is_in_loop():
         raise HTTPException(
             status_code=400, detail="Gravity control is already running"
         )
@@ -923,12 +925,12 @@ async def start_gravity(
             status_code=400, detail="Gravity compensation is only for SO-100 robot"
         )
 
-    gravity_control.start()
+    signal_gravity_control.start()
 
     # Add background task to run the control loop
     background_tasks.add_task(
         background_task_log_exceptions(robot.gravity_compensation_loop),
-        control_signal=gravity_control,
+        control_signal=signal_gravity_control,
     )
     return StatusResponse()
 
@@ -945,10 +947,10 @@ async def stop_gravity_compensation(
     """
     Stop the gravity compensation for all robots.
     """
-    if not gravity_control.is_in_loop():
+    if not signal_gravity_control.is_in_loop():
         return StatusResponse(status="error", message="Gravity control is not running")
 
-    gravity_control.stop()
+    signal_gravity_control.stop()
     return StatusResponse(message="Stopping gravity control")
 
 
@@ -958,88 +960,11 @@ async def stop_gravity_compensation(
     summary="Get the status of the auto control by AI",
     description="Get the status of the auto control by AI.",
 )
-async def fetch_auto_control_status(request: AIStatusRequest) -> AIStatusResponse:
+async def fetch_auto_control_status() -> AIStatusResponse:
     """
     Fetch the status of the auto control by AI
     """
-    supabase_id: str | None = None
-    supabase_status: Literal["stopped", "running", "paused", "waiting"] | None = None
-
-    supabase_client = await get_client()
-    try:
-        user = await supabase_client.auth.get_user()
-    except Exception as e:
-        logger.warning(f"Failed to loggin: {e}")
-        raise HTTPException(status_code=401, detail="Not authenticated")
-
-    if user is not None:
-        supabase_response = (
-            await supabase_client.table("ai_control_sessions")
-            .select("*, servers(status)")
-            .eq("user_id", user.user.id)
-            .order("created_at", desc=True)
-            .limit(1)
-            .execute()
-        )
-        if supabase_response.data:
-            supabase_data = supabase_response.data[0]
-            supabase_id = supabase_data["id"]
-            supabase_status = supabase_data["status"]
-            # If server status is stopped, set status to stopped
-            if (
-                supabase_data["servers"] is not None
-                and supabase_data["servers"]["status"] == "stopped"
-            ):
-                # Set the backend status to stopped
-                supabase_status = "stopped"
-                # Update the status in the database
-                await (
-                    supabase_client.table("ai_control_sessions")
-                    .update({"status": supabase_status})
-                    .eq("id", supabase_id)
-                    .execute()
-                )
-            # If ai-control signal is stopped but remote status is running, set the remote status to stopped
-            if ai_control_signal.status == "stopped" and supabase_status == "running":
-                supabase_status = "stopped"
-                # Update the status in the database
-                await (
-                    supabase_client.table("ai_control_sessions")
-                    .update({"status": supabase_status})
-                    .eq("id", supabase_id)
-                    .execute()
-                )
-
-    # Situation 1: There is already a different, running process in backend
-    if (
-        supabase_id is not None
-        and supabase_id != ai_control_signal.id
-        and (
-            supabase_status == "running"
-            or supabase_status == "paused"
-            or supabase_status == "waiting"
-        )
-    ):
-        # if started less than 10 minutes ago, return the backend status
-        created_at = parser.isoparse(supabase_data["created_at"])
-        if (
-            datetime.datetime.now(datetime.timezone.utc) - created_at
-        ).total_seconds() < 600:
-            ai_control_signal.id = supabase_id
-            return AIStatusResponse(id=supabase_id, status=supabase_status)
-
-    # Situation 2: The backend says the process should be waiting or stopped
-    if (
-        supabase_id is not None
-        and supabase_id == ai_control_signal.id
-        and supabase_status == "stopped"
-    ):
-        # Stop the local process
-        ai_control_signal.status = supabase_status
-        return AIStatusResponse(id=supabase_id, status=supabase_status)
-
-    # Situation 3: return the current local status
-    return AIStatusResponse(id=ai_control_signal.id, status=ai_control_signal.status)
+    return AIStatusResponse(id=signal_ai_control.id, status=signal_ai_control.status)
 
 
 @router.post(
@@ -1086,7 +1011,7 @@ async def spawn_inference_server(
         model_id=query.model_id,
         init_connected_robots=False,
         model_type=query.model_type,
-        ai_control_signal_id=ai_control_signal.id,
+        ai_control_signal_id=signal_ai_control.id,
     )
 
     return SpawnStatusResponse(message="ok", server_info=server_info)
@@ -1098,7 +1023,7 @@ async def spawn_inference_server(
     summary="Start the auto control by AI",
     description="Start the auto control by AI.",
 )
-async def start_auto_control(
+async def start_ai_control(
     query: StartAIControlRequest,
     background_tasks: BackgroundTasks,
     rcm: RobotConnectionManager = Depends(get_rcm),
@@ -1108,17 +1033,29 @@ async def start_auto_control(
     """
     Start the auto control by AI
     """
-    if ai_control_signal.is_in_loop():
+
+    if signal_leader_follower.is_in_loop():
+        raise HTTPException(
+            status_code=400,
+            detail="Leader-follower control is running. Stop it before starting AI control.",
+        )
+    if signal_gravity_control.is_in_loop():
+        raise HTTPException(
+            status_code=400,
+            detail="Gravity compensation is running. Stop it before starting AI control.",
+        )
+
+    if signal_ai_control.is_in_loop():
         return AIControlStatusResponse(
             status="error",
             message="Auto control is already running",
-            ai_control_signal_id=ai_control_signal.id,
-            ai_control_signal_status=ai_control_signal.status,
+            ai_control_signal_id=signal_ai_control.id,
+            ai_control_signal_status=signal_ai_control.status,
             server_info=None,
         )
 
-    ai_control_signal.new_id()
-    ai_control_signal.start()
+    signal_ai_control.new_id()
+    signal_ai_control.start()
 
     supabase_client = await get_client()
     user = await supabase_client.auth.get_user()
@@ -1128,12 +1065,13 @@ async def start_auto_control(
         supabase_client.table("ai_control_sessions")
         .upsert(
             {
-                "id": ai_control_signal.id,
+                "id": signal_ai_control.id,
                 "user_id": user.user.id,
                 "user_email": user.user.email,
                 "model_type": query.model_type,
                 "model_id": query.model_id,
                 "prompt": query.prompt,
+                "checkpoint": query.checkpoint,
                 "status": "waiting",
             }
         )
@@ -1165,8 +1103,9 @@ async def start_auto_control(
         model_type=query.model_type,
         model_id=query.model_id,
         cameras_keys_mapping=query.cameras_keys_mapping,
-        ai_control_signal_id=ai_control_signal.id,
+        ai_control_signal_id=signal_ai_control.id,
         verify_cameras=query.verify_cameras,
+        checkpoint=query.checkpoint,
     )
 
     # Add a flag: successful setup
@@ -1178,14 +1117,14 @@ async def start_auto_control(
                 "server_id": server_info.server_id,
             }
         )
-        .eq("id", ai_control_signal.id)
+        .eq("id", signal_ai_control.id)
         .execute()
     )
 
     background_tasks.add_task(
         model.control_loop,
         robots=robots_to_control,
-        control_signal=ai_control_signal,
+        control_signal=signal_ai_control,
         prompt=query.prompt,
         all_cameras=all_cameras,
         model_spawn_config=model_spawn_config,
@@ -1197,9 +1136,9 @@ async def start_auto_control(
 
     return AIControlStatusResponse(
         status="ok",
-        message=f"Starting AI control with id: {ai_control_signal.id}",
+        message=f"Starting AI control with id: {signal_ai_control.id}",
         server_info=server_info,
-        ai_control_signal_id=ai_control_signal.id,
+        ai_control_signal_id=signal_ai_control.id,
         ai_control_signal_status="waiting",
     )
 
@@ -1210,17 +1149,37 @@ async def start_auto_control(
     summary="Stop the auto control by AI",
     description="Stop the auto control by AI.",
 )
-async def stop_auto_control(
+async def stop_ai_control(
+    background_tasks: BackgroundTasks,
     rcm: RobotConnectionManager = Depends(get_rcm),
+    session=Depends(user_is_logged_in),
 ) -> StatusResponse:
     """
     Stop the auto control by AI
     """
-    if not ai_control_signal.is_in_loop():
+
+    tokens = get_tokens()
+
+    # Call the /stop endpoint in Modal
+    @background_task_log_exceptions
+    async def stop_modal():
+        async with httpx.AsyncClient(timeout=5) as client:
+            await client.post(
+                url=f"{tokens.MODAL_API_URL}/stop",
+                headers={
+                    "Authorization": f"Bearer {session.access_token}",
+                    "Content-Type": "application/json",
+                },
+            )
+
+    background_tasks.add_task(stop_modal)
+
+    if not signal_ai_control.is_in_loop():
         return StatusResponse(message="Auto control is not running")
 
-    ai_control_signal.stop()
-    return StatusResponse(message="Stopping auto control")
+    signal_ai_control.stop()
+
+    return StatusResponse(message="Stopped AI control")
 
 
 @router.post(
@@ -1229,17 +1188,17 @@ async def stop_auto_control(
     summary="Pause the auto control by AI",
     description="Pause the auto control by AI.",
 )
-async def pause_auto_control(
+async def pause_ai_control(
     rcm: RobotConnectionManager = Depends(get_rcm),
 ) -> StatusResponse:
     """
     Pause the auto control by AI
     """
-    if not ai_control_signal.is_in_loop():
+    if not signal_ai_control.is_in_loop():
         return StatusResponse(message="Auto control is not running")
 
-    ai_control_signal.status = "paused"
-    return StatusResponse(message="Pausing auto control")
+    signal_ai_control.status = "paused"
+    return StatusResponse(message="Pausing AI control")
 
 
 @router.post(
@@ -1248,17 +1207,17 @@ async def pause_auto_control(
     summary="Resume the auto control by AI",
     description="Resume the auto control by AI.",
 )
-async def resume_auto_control(
+async def resume_ai_control(
     rcm: RobotConnectionManager = Depends(get_rcm),
 ) -> StatusResponse:
     """
     Resume the auto control by AI
     """
-    if ai_control_signal.status == "running":
-        return StatusResponse(message="Auto control is already running")
+    if signal_ai_control.status == "running":
+        return StatusResponse(message="AI control is already running")
 
-    ai_control_signal.status = "running"
-    return StatusResponse(message="Resuming auto control")
+    signal_ai_control.status = "running"
+    return StatusResponse(message="Resuming AI control")
 
 
 @router.post(
@@ -1266,7 +1225,7 @@ async def resume_auto_control(
     response_model=StatusResponse,
     summary="Feedback about the auto control session",
 )
-async def feedback_auto_control(
+async def feedback_ai_control(
     request: FeedbackRequest,
     session=Depends(user_is_logged_in),
 ) -> StatusResponse:
@@ -1277,11 +1236,7 @@ async def feedback_auto_control(
 
     await (
         supabase_client.table("ai_control_sessions")
-        .update(
-            {
-                "feedback": request.feedback,
-            }
-        )
+        .update({"feedback": request.feedback})
         .eq("id", request.ai_control_id)
         .execute()
     )
