@@ -67,6 +67,14 @@ class RobotStatus(BaseModel):
     nb_actions_received: int
 
 
+class EndEffectorReadRequest(BaseModel):
+    sync: bool = Field(
+        False,
+        description="If True, the simulation will first read the motor positions, synchronize them with the simulated robot, and then return the end effector position."
+        + "Useful for measurements, however it will take more time to respond.",
+    )
+
+
 class EndEffectorPosition(BaseModel):
     """
     End effector position for a movement in absolute frame.
@@ -711,7 +719,7 @@ class DeleteEpisodeRequest(BaseModel):
     episode_id: int
 
 
-class ModelVideoKeysRequest(BaseModel):
+class ModelConfigurationRequest(BaseModel):
     model_id: str = Field(
         ...,
         description="Hugging Face model id to use",
@@ -725,12 +733,17 @@ class ModelVideoKeysRequest(BaseModel):
     )
 
 
-class ModelVideoKeysResponse(BaseModel):
+class ModelConfigurationResponse(BaseModel):
     video_keys: List[str] = Field(
         ...,
         description="List of video keys for the model. These are the keys used to access the videos in the dataset.",
         examples=[["video_0", "video_1"]],
     )
+    checkpoints: List[str] = Field(
+        default_factory=list,
+        description="List of available checkpoints for the model.",
+        examples=[["100", "500"]],
+    )
 
 
 class AdminSettingsRequest(BaseModel):
@@ -869,11 +882,10 @@ class StartAIControlRequest(BaseModel):
         True,
         description="Whether to verify the setup before starting the AI control. If False, skips the verification step.",
     )
-
-
-class AIStatusRequest(BaseModel):
-    user_id: str = Field(
-        ..., description="User ID of the user who started the AI control"
+    checkpoint: int | None = Field(
+        None,
+        description="Checkpoint to use for the model. If None, uses the latest checkpoint.",
+        examples=[500],
     )
 
 
@@ -981,13 +993,13 @@ class RobotPairRequest(BaseModel):
     Represents a pair of robots for leader-follower control.
     """
 
+    model_config = ConfigDict(extra="ignore")
+
     leader_id: int | None = Field(..., description="Serial number of the leader robot")
     follower_id: int | None = Field(
         ..., description="Serial number of the follower robot"
     )
 
-    model_config = ConfigDict(extra="ignore")
-
 
 class StartLeaderArmControlRequest(BaseModel):
     """
